package xmlunleashed.ch10;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

import java.math.BigDecimal;

import xmlunleashed.ch10.jaxb.RentalProperty;
import xmlunleashed.ch10.jaxb.RentalPropertyList;
import xmlunleashed.ch10.jaxb.Contact;
import xmlunleashed.ch10.jaxb.Address;


/**
 *  Data Access Object (DAO) for the rental_property database. <br>
 *
 */
public class RentalPropertyDAO {

	/**
	 *  The database connection
	 */
	protected Connection myConn;

	/**
	 *  Constructor for DAO.  Setup the database connection. Use the default properties.
	 */
	public RentalPropertyDAO()
		throws DAOException {
			this("sun.jdbc.odbc.JdbOdbcDriver", "jdbc:odbc:RentalPropertyDSN", "test", "test");
	}

	/**
	 *  Constructor for DAO.  Setup the database connection.
	 */
	public RentalPropertyDAO(String driverName, String dbUrl, String user, String pass)
		throws DAOException {

		try {
			// Load the driver
			log("Loading driver: "  + driverName);
			Class.forName(driverName);

			// Get a connection
			log("Connecting to the database: "  + dbUrl);
			log("User id: "  + user);
			myConn = DriverManager.getConnection (dbUrl, user, pass);

			log("DB connection successful at " + new java.util.Date());
		}
		catch (Exception exc) {
			throw new DAOException(exc);
		}
	}

	/**
	 *  Get a list of rental properties from the database
	 *
	 *  @return a list of RentalProperty objects
	 *  @exception SQLException thrown for SQL errors
	 */
	public RentalPropertyList getRentalProperties() throws DAOException {

		RentalPropertyList theRentalPropertyList = new RentalPropertyList();
		java.util.List theList = theRentalPropertyList.getList();

		try {
			Statement myStmt = myConn.createStatement();

			String rentalSql = "SELECT prop_num, name, street_address, city, state, zip_code, size_sq, bed_count, bath_count, monthly_rent, voice_phone, fax_phone FROM rental_properties";

			ResultSet myRs = myStmt.executeQuery(rentalSql);

			RentalProperty tempProperty = null;

			// build a collection of JAXB RentalProperty objects
			while (myRs.next()) {
				tempProperty = createRentalProperty(myRs);
				theList.add(tempProperty);
			}

			// be sure to validate the new list
			theRentalPropertyList.validate();

			myRs.close();
			myStmt.close();
		}
		catch (Exception exc) {
			throw new DAOException(exc);
		}

		return theRentalPropertyList;
	}

	/**
	 *  Create a JAXB RentalProperty object based on the result set.
	 *  This method provides the mapping between database schema and object
	 */
	protected RentalProperty createRentalProperty(ResultSet theRs) throws DAOException {

		RentalProperty theProperty = new RentalProperty();
		Address theAddress = new Address();
		Contact theContact = new Contact();

		try {
			// set the rental property number and name
			theProperty.setPropId(theRs.getString("prop_num"));
			theProperty.setName(theRs.getString("name"));

			// set the address
			theAddress.setStreet(theRs.getString("street_address"));
			theAddress.setCity(theRs.getString("city"));
			theAddress.setState(theRs.getString("state"));
			theAddress.setPostalCode(theRs.getString("zip_code"));
			theProperty.setAddress(theAddress);

			// set the square footage, bedrooms, bath count and rent
			theProperty.setSquareFootage(theRs.getDouble("size_sq"));
			theProperty.setBedrooms(theRs.getDouble("bed_count"));
			theProperty.setBath(theRs.getDouble("bath_count"));
			theProperty.setPrice(new BigDecimal(theRs.getDouble("monthly_rent")));

			// set the contact information
			theContact.setPhone(theRs.getString("voice_phone"));
			theContact.setFax(theRs.getString("fax_phone"));
			theProperty.setContact(theContact);
		}
		catch (SQLException exc) {
			throw new DAOException(exc);
		}

		return theProperty;
	}

	/**
	 *  Utility method for logging
	 */
	protected void log(Object message) {
		System.out.println("RentalPropertyDAO: " + message);
	}
}